"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JourneyLanguageServiceHost = void 0;
const ts = require("typescript");
const staticFiles_1 = require("./staticFiles");
class JourneyLanguageServiceHost {
    constructor(fs, config, customTransformers) {
        var _a;
        this.readFile = (fileName) => {
            var _a, _b;
            return (_b = (_a = this.fs.read(fileName)) === null || _a === void 0 ? void 0 : _a.content) !== null && _b !== void 0 ? _b : undefined;
        };
        this.fileExists = (fileName) => {
            return this.fs.info(fileName) != null;
        };
        this.readDirectory = (path, extensions, exclude, include, depth) => {
            // Used for auto-completing import paths
            if (path == '') {
                // Avoid returning anything here
                return [];
            }
            const files = this.getModuleFileNames().filter((filename) => {
                if (!filename.startsWith(path)) {
                    return false;
                }
                const rest = filename.substring(path.length);
                const fileDepth = rest.split('/').length;
                if (fileDepth > (depth !== null && depth !== void 0 ? depth : 1)) {
                    return false;
                }
                if (extensions) {
                    const matched = extensions.filter((ext) => filename.endsWith(ext));
                    if (matched.length == 0) {
                        return false;
                    }
                }
                return true;
            });
            return files;
        };
        this.getDirectories = (directoryName) => {
            // Used for auto-completing import paths
            if (directoryName == '') {
                // Special case to avoid returning "file:"
                return [];
            }
            const scripts = this.getModuleFileNames().filter((filename) => filename.startsWith(directoryName));
            let dirs = {};
            for (let script of scripts) {
                const rest = script.substring(directoryName.length);
                const components = rest.split('/');
                if (components.length > 1 && components[0] != '') {
                    dirs[components[0]] = true;
                }
            }
            return Object.keys(dirs);
        };
        this.fs = fs;
        this.compilerOptions = config.compilerOptions;
        this.config = config;
        this.customTransformers = customTransformers;
        this.defaultLibLocation = (_a = config.defaultLibLocation) !== null && _a !== void 0 ? _a : staticFiles_1.DEFAULT_LIB_LOCATION;
    }
    // --- language service host ---------------
    getCompilationSettings() {
        return this.compilerOptions;
    }
    getScriptFileNames() {
        return this.fs.listFiles().filter((path) => {
            const info = this.fs.info(path);
            return info === null || info === void 0 ? void 0 : info.isScript;
        });
    }
    getScriptVersion(fileName) {
        var _a;
        const info = this.fs.info(fileName);
        return (_a = info === null || info === void 0 ? void 0 : info.version) !== null && _a !== void 0 ? _a : '';
    }
    getScriptSnapshot(fileName) {
        const node = this.fs.read(fileName);
        if (node == null) {
            return undefined;
        }
        return ts.ScriptSnapshot.fromString(node.content);
    }
    getModuleFileNames() {
        return this.fs.listFiles().filter((path) => {
            const info = this.fs.info(path);
            return info === null || info === void 0 ? void 0 : info.isModule;
        });
    }
    getScriptKind(fileName) {
        const suffix = fileName.substr(fileName.lastIndexOf('.') + 1);
        switch (suffix) {
            case 'ts':
                return ts.ScriptKind.TS;
            case 'tsx':
                return ts.ScriptKind.TSX;
            case 'js':
                return ts.ScriptKind.JS;
            case 'jsx':
                return ts.ScriptKind.JSX;
            default:
                return this.getCompilationSettings().allowJs ? ts.ScriptKind.JS : ts.ScriptKind.TS;
        }
    }
    getCurrentDirectory() {
        return '';
    }
    getDefaultLibFileName(options) {
        return `${this.defaultLibLocation}${ts.getDefaultLibFileName(options)}`;
    }
    resolveTypeReferenceDirectives(typeDirectiveNames, containingFile, redirectedReference, options) {
        let resolved = [];
        const resolutionHost = this;
        for (let directive of typeDirectiveNames) {
            const result = ts.resolveTypeReferenceDirective(directive, containingFile, options, resolutionHost, redirectedReference);
            resolved.push(result === null || result === void 0 ? void 0 : result.resolvedTypeReferenceDirective);
        }
        return resolved;
    }
    resolveModuleNames(moduleNames, containingFile, reusedNames, redirectedReference, options) {
        const resolvedModules = [];
        const resolutionHost = this;
        for (const moduleName of moduleNames) {
            // try to use standard resolution
            const result = ts.resolveModuleName(moduleName, containingFile, this.compilerOptions, resolutionHost, undefined, redirectedReference);
            if (result.resolvedModule) {
                resolvedModules.push(result.resolvedModule);
                continue;
            }
            else if (moduleName.startsWith('.')) {
                // Relative path, consider not found - an error
            }
            else {
                if (this.config.requireHack) {
                    // Try with classic
                    const classicResult = ts.classicNameResolver(moduleName, containingFile, this.compilerOptions, resolutionHost);
                    if (classicResult.resolvedModule) {
                        resolvedModules.push(classicResult.resolvedModule);
                        continue;
                    }
                }
                // The module could not be found.
                // These generate TS2307 errors, which we then filter out before presenting errors to the user.
            }
            resolvedModules.push(undefined);
        }
        return resolvedModules;
    }
    getCustomTransformers() {
        return this.customTransformers;
    }
    mapDefinitions(defs) {
        var _a;
        return ((_a = defs === null || defs === void 0 ? void 0 : defs.map((def) => {
            return this.mapDefinition(def);
        }).filter((def) => def != null)) !== null && _a !== void 0 ? _a : []);
    }
    mapDefinition(def) {
        if (def == null) {
            return null;
        }
        const path = def.fileName;
        const info = this.fs.info(path);
        if (info == null || !info.generated) {
            // Direct source file
            return def;
        }
        const node = this.fs.read(path);
        if (node == null) {
            // Cannot find file
            return null;
        }
        const mapped = node.mapTargetToSource(def.textSpan);
        if (mapped) {
            return Object.assign(Object.assign({}, def), { fileName: mapped.path, textSpan: {
                    start: mapped.start,
                    length: mapped.length
                }, originalFileName: def.fileName, originalTextSpan: def.textSpan });
        }
        else {
            // Cannot find this specific definition in a source file
            return null;
        }
    }
}
exports.JourneyLanguageServiceHost = JourneyLanguageServiceHost;
//# sourceMappingURL=JourneyLanguageServiceHost.js.map